/*
 * Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Radek Novacek <rnovacek@redhat.com>
 */

#include <konkret/konkret.h>
#include "LMI_EthernetPort.h"

#include "network.h"
#include "globals.h"
#include "port.h"

static const CMPIBroker* _cb = NULL;

static void LMI_EthernetPortInitialize(CMPIInstanceMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus LMI_EthernetPortCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_EthernetPortEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus LMI_EthernetPortEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    Network *network = mi->hdl;
    const char *ns = KNameSpace(cop);

    network_lock(network);
    const Ports *ports = network_get_ports(network);
    Port *port;
    for (size_t i = 0; i < ports_length(ports); ++i) {
        port = ports_index(ports, i);

        LMI_EthernetPort w;
        LMI_EthernetPort_Init(&w, _cb, ns);

        LMI_EthernetPort_Set_SystemName(&w, get_system_name());
        LMI_EthernetPort_Set_CreationClassName(&w, "LMI_EthernetPort");
        LMI_EthernetPort_Set_SystemCreationClassName(&w, get_system_creationg_class_name());
        LMI_EthernetPort_Set_DeviceID(&w, port_get_id(port));

        switch (port_get_state(port)) {
            case STATE_NA:
                LMI_EthernetPort_Set_EnabledState(&w, LMI_EthernetPort_EnabledState_Not_Applicable);
                break;
            case STATE_OFFLINE:
                LMI_EthernetPort_Set_EnabledState(&w, LMI_EthernetPort_EnabledState_Enabled_but_Offline);
                break;
            case STATE_SERVICING:
                LMI_EthernetPort_Set_EnabledState(&w, LMI_EthernetPort_EnabledState_Enabled);
                break;
            case STATE_STARTING:
                LMI_EthernetPort_Set_EnabledState(&w, LMI_EthernetPort_EnabledState_Starting);
                break;
            case STATE_STOPPING:
                LMI_EthernetPort_Set_EnabledState(&w, LMI_EthernetPort_EnabledState_Shutting_Down);
                break;
            case STATE_STOPPED:
                LMI_EthernetPort_Set_EnabledState(&w, LMI_EthernetPort_EnabledState_Disabled);
                break;
            case STATE_ABORTED:
                LMI_EthernetPort_Set_EnabledState(&w, LMI_EthernetPort_EnabledState_Disabled);
                break;
            default:
                LMI_EthernetPort_Set_EnabledState(&w, LMI_EthernetPort_EnabledState_Unknown);
        }
        LMI_EthernetPort_Init_AvailableRequestedStates(&w, 2);
        LMI_EthernetPort_Set_AvailableRequestedStates(&w, 0, LMI_EthernetPort_AvailableRequestedStates_Enabled);
        LMI_EthernetPort_Set_AvailableRequestedStates(&w, 1, LMI_EthernetPort_AvailableRequestedStates_Disabled);
        KReturnInstance(cr, w);
    }
    network_unlock(network);
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_EthernetPortGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus LMI_EthernetPortCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_EthernetPortModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char** properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_EthernetPortDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_EthernetPortExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

CMInstanceMIStub(
    LMI_EthernetPort,
    LMI_EthernetPort,
    _cb,
    LMI_EthernetPortInitialize(&mi))

static void LMI_EthernetPortMethodInitialize(CMPIMethodMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus LMI_EthernetPortMethodCleanup(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_EthernetPortInvokeMethod(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* meth,
    const CMPIArgs* in,
    CMPIArgs* out)
{
    return LMI_EthernetPort_DispatchMethod(
        _cb, mi, cc, cr, cop, meth, in, out);
}

CMMethodMIStub(
    LMI_EthernetPort,
    LMI_EthernetPort,
    _cb,
    LMI_EthernetPortMethodInitialize(&mi))

KUint32 LMI_EthernetPort_RequestStateChange(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const LMI_EthernetPortRef* self,
    const KUint16* RequestedState,
    KRef* Job,
    const KDateTime* TimeoutPeriod,
    CMPIStatus* status)
{
    Network *network = mi->hdl;
    KUint32 result = KUINT32_INIT;
    GError *err = NULL;

    Port *port = NULL;
    const Ports *ports = network_get_ports(network);
    for (size_t i = 0; i < ports_length(ports); ++i) {
        if (strcmp(port_get_id(ports_index(ports, i)), self->DeviceID.chars) == 0) {
            port = ports_index(ports, i);
        }
    }

    Require(RequestedState, "No state has been requested");

    switch (RequestedState->value) {
        case LMI_EthernetPort_RequestedState_Disabled:
            port_set_state(port, STATE_STOPPED, &err);
            break;
        case LMI_EthernetPort_RequestedState_Enabled:
            port_set_state(port, STATE_SERVICING, &err);
            break;
        default:
            KSetStatus2(_cb, status, ERR_INVALID_PARAMETER, "Invalid state requested");
            KUint32_Set(&result, 1);
            return result;
    }

    if (err != NULL) {
        error(err->message);
        KSetStatus2(_cb, status, ERR_FAILED, err->message);
        KUint32_Set(&result, err->code);
        return result;
    }

    KSetStatus(status, OK);
    KUint32_Set(&result, 0);
    return result;
}

KUint32 LMI_EthernetPort_SetPowerState(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const LMI_EthernetPortRef* self,
    const KUint16* PowerState,
    const KDateTime* Time,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 LMI_EthernetPort_Reset(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const LMI_EthernetPortRef* self,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 LMI_EthernetPort_EnableDevice(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const LMI_EthernetPortRef* self,
    const KBoolean* Enabled,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 LMI_EthernetPort_OnlineDevice(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const LMI_EthernetPortRef* self,
    const KBoolean* Online,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 LMI_EthernetPort_QuiesceDevice(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const LMI_EthernetPortRef* self,
    const KBoolean* Quiesce,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 LMI_EthernetPort_SaveProperties(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const LMI_EthernetPortRef* self,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KUint32 LMI_EthernetPort_RestoreProperties(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const LMI_EthernetPortRef* self,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KONKRET_REGISTRATION(
    "root/cimv2",
    "LMI_EthernetPort",
    "LMI_EthernetPort",
    "instance method")
