/*
 * Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Radek Novacek <rnovacek@redhat.com>
 */

#include "globals.h"
#include <konkret/konkret.h>
#include "LMI_OrderedComponent.h"
#include <LMI_DNSSettingData.h>
#include "network.h"
#include "connection.h"
#include "ref_factory.h"
#include "setting.h"

static const CMPIBroker* _cb;

static void LMI_OrderedComponentInitialize(CMPIInstanceMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus LMI_OrderedComponentCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_OrderedComponentEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus LMI_OrderedComponentEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    Network *network = mi->hdl;
    const char *ns = KNameSpace(cop);

    char *instanceID;
    size_t j;
    Setting *setting;
    Connection *connection;
    network_lock(network);
    const Connections *connections = network_get_connections(network);
    if (connections == NULL) {
        network_unlock(network);
        CMReturn(CMPI_RC_OK);
    }
    for (size_t i = 0; i < connections_length(connections); ++i) {
        connection = connections_index(connections, i);

        CMPIObjectPath *groupComponentOP = connectionToCIM_IPAssignmentSettingDataRefOP(connection, _cb, ns);

        for (j = 0; j < settings_length(connection_get_settings(connection)); ++j) {
            setting = settings_index(connection_get_settings(connection), j);

            CMPIObjectPath *partComponentOP = settingToLMI_IPAssignmentSettingDataSubclassOP(setting, _cb, ns);

            LMI_OrderedComponent w;
            LMI_OrderedComponent_Init(&w, _cb, ns);
            LMI_OrderedComponent_SetObjectPath_GroupComponent(&w, groupComponentOP);
            LMI_OrderedComponent_SetObjectPath_PartComponent(&w, partComponentOP);
            LMI_OrderedComponent_Set_AssignedSequence(&w, 1);

            KReturnInstance(cr, w);

            LMI_DNSSettingDataRef dnsSettingDataRef;
            LMI_DNSSettingDataRef_Init(&dnsSettingDataRef, _cb, ns);
            asprintf(&instanceID, "%s_%ld", connection_get_id(connection), j);
            LMI_DNSSettingDataRef_Set_InstanceID(&dnsSettingDataRef, instanceID);
            free(instanceID);

            CMPIStatus rc;
            CMPIObjectPath *dnsSettingDataOP = LMI_DNSSettingDataRef_ToObjectPath(&dnsSettingDataRef, &rc);
            LMI_OrderedComponent_SetObjectPath_PartComponent(&w, dnsSettingDataOP);
            KReturnInstance(cr, w);
        }
    }
    network_unlock(network);
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_OrderedComponentGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus LMI_OrderedComponentCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_OrderedComponentModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char**properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_OrderedComponentDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_OrderedComponentExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static void LMI_OrderedComponentAssociationInitialize(CMPIAssociationMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus LMI_OrderedComponentAssociationCleanup(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_OrderedComponentAssociators(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole,
    const char** properties)
{
    return KDefaultAssociators(
        _cb,
        mi,
        cc,
        cr,
        cop,
        LMI_OrderedComponent_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole,
        properties);
}

static CMPIStatus LMI_OrderedComponentAssociatorNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* resultClass,
    const char* role,
    const char* resultRole)
{
    return KDefaultAssociatorNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        LMI_OrderedComponent_ClassName,
        assocClass,
        resultClass,
        role,
        resultRole);
}

static CMPIStatus LMI_OrderedComponentReferences(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role,
    const char** properties)
{
    return KDefaultReferences(
        _cb,
        mi,
        cc,
        cr,
        cop,
        LMI_OrderedComponent_ClassName,
        assocClass,
        role,
        properties);
}

static CMPIStatus LMI_OrderedComponentReferenceNames(
    CMPIAssociationMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* assocClass,
    const char* role)
{
    return KDefaultReferenceNames(
        _cb,
        mi,
        cc,
        cr,
        cop,
        LMI_OrderedComponent_ClassName,
        assocClass,
        role);
}

CMInstanceMIStub(
    LMI_OrderedComponent,
    LMI_OrderedComponent,
    _cb,
    LMI_OrderedComponentInitialize(&mi))

CMAssociationMIStub(
    LMI_OrderedComponent,
    LMI_OrderedComponent,
    _cb,
    LMI_OrderedComponentAssociationInitialize(&mi))

KONKRET_REGISTRATION(
    "root/cimv2",
    "LMI_OrderedComponent",
    "LMI_OrderedComponent",
    "instance association")
