/*
 * Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Radek Novacek <rnovacek@redhat.com>
 */

#include "globals.h"
#include <konkret/konkret.h>
#include "LMI_RemoteServiceAccessPoint.h"
#include "network.h"
#include "activeconnection.h"
#include "connection.h"
#include "setting.h"
#include "endpoint.h"
#include "port.h"

static const CMPIBroker* _cb = NULL;

static void LMI_RemoteServiceAccessPointInitialize(CMPIInstanceMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus LMI_RemoteServiceAccessPointCleanup(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_RemoteServiceAccessPointEnumInstanceNames(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    return KDefaultEnumerateInstanceNames(
        _cb, mi, cc, cr, cop);
}

static CMPIStatus LMI_RemoteServiceAccessPointEnumInstances(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    Network *network = mi->hdl;
    const char *ns = KNameSpace(cop);

    size_t j;
    Port *port;
    Endpoint *endpoint;
    const char *name;
    char *n;

    network_lock(network);
    const Ports *ports = network_get_ports(network);
    for (size_t i = 0; i < ports_length(ports); ++i) {
        port = ports_index(ports, i);
        for (j = 0; j < endpoints_length(port_get_endpoints(port)); ++j) {
            endpoint = endpoints_index(port_get_endpoints(port), j);

            if (endpoint_get_type(endpoint) == Unknown || (endpoint_get_ipv4gateway(endpoint) == NULL && endpoint_get_ipv6gateway(endpoint) == NULL)) {
                continue;
            }

            name = endpoint_get_name(endpoint);
            LMI_RemoteServiceAccessPoint w;
            LMI_RemoteServiceAccessPoint_Init(&w, _cb, ns);
            LMI_RemoteServiceAccessPoint_Set_CreationClassName(&w, "LMI_RemoteServiceAccessPoint");
            LMI_RemoteServiceAccessPoint_Set_Name(&w, name);
            LMI_RemoteServiceAccessPoint_Set_SystemCreationClassName(&w, get_system_creationg_class_name());
            LMI_RemoteServiceAccessPoint_Set_SystemName(&w, get_system_name());
            LMI_RemoteServiceAccessPoint_Set_AccessContext(&w, LMI_RemoteServiceAccessPoint_AccessContext_Default_Gateway);
            switch (endpoint_get_type(endpoint)) {
                case IPv4:
                    LMI_RemoteServiceAccessPoint_Set_AccessInfo(&w, endpoint_get_ipv4gateway(endpoint));
                    LMI_RemoteServiceAccessPoint_Set_InfoFormat(&w, LMI_RemoteServiceAccessPoint_InfoFormat_IPv4_Address);
                    KReturnInstance(cr, w);
                    break;
                case IPv6:
                    LMI_RemoteServiceAccessPoint_Set_AccessInfo(&w, endpoint_get_ipv6gateway(endpoint));
                    LMI_RemoteServiceAccessPoint_Set_InfoFormat(&w, LMI_RemoteServiceAccessPoint_InfoFormat_IPv6_Address);
                    KReturnInstance(cr, w);
                    break;
                case IPv4_IPv6:
                    // TODO: test it
                    asprintf(&n, "%s IPv4", name);
                    LMI_RemoteServiceAccessPoint_Set_Name(&w, n);
                    LMI_RemoteServiceAccessPoint_Set_AccessInfo(&w, endpoint_get_ipv4gateway(endpoint));
                    LMI_RemoteServiceAccessPoint_Set_InfoFormat(&w, LMI_RemoteServiceAccessPoint_InfoFormat_IPv4_Address);
                    KReturnInstance(cr, w);
                    free(n);

                    asprintf(&n, "%s IPv6", name);
                    LMI_RemoteServiceAccessPoint_Set_Name(&w, n);
                    LMI_RemoteServiceAccessPoint_Set_AccessInfo(&w, endpoint_get_ipv6gateway(endpoint));
                    LMI_RemoteServiceAccessPoint_Set_InfoFormat(&w, LMI_RemoteServiceAccessPoint_InfoFormat_IPv6_Address);
                    KReturnInstance(cr, w);
                    free(n);
                    break;
                default:
                    break;
            }
        }
    }
    network_unlock(network);
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_RemoteServiceAccessPointGetInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char** properties)
{
    return KDefaultGetInstance(
        _cb, mi, cc, cr, cop, properties);
}

static CMPIStatus LMI_RemoteServiceAccessPointCreateInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_RemoteServiceAccessPointModifyInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const CMPIInstance* ci,
    const char** properties)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_RemoteServiceAccessPointDeleteInstance(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

static CMPIStatus LMI_RemoteServiceAccessPointExecQuery(
    CMPIInstanceMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* lang,
    const char* query)
{
    CMReturn(CMPI_RC_ERR_NOT_SUPPORTED);
}

CMInstanceMIStub(
    LMI_RemoteServiceAccessPoint,
    LMI_RemoteServiceAccessPoint,
    _cb,
    LMI_RemoteServiceAccessPointInitialize(&mi))

static void LMI_RemoteServiceAccessPointMethodInitialize(CMPIMethodMI *mi)
{
    mi->hdl = network_ref(_cb);
}

static CMPIStatus LMI_RemoteServiceAccessPointMethodCleanup(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    CMPIBoolean term)
{
    network_unref(mi->hdl);
    mi->hdl = NULL;
    CMReturn(CMPI_RC_OK);
}

static CMPIStatus LMI_RemoteServiceAccessPointInvokeMethod(
    CMPIMethodMI* mi,
    const CMPIContext* cc,
    const CMPIResult* cr,
    const CMPIObjectPath* cop,
    const char* meth,
    const CMPIArgs* in,
    CMPIArgs* out)
{
    return LMI_RemoteServiceAccessPoint_DispatchMethod(
        _cb, mi, cc, cr, cop, meth, in, out);
}

CMMethodMIStub(
    LMI_RemoteServiceAccessPoint,
    LMI_RemoteServiceAccessPoint,
    _cb,
    LMI_RemoteServiceAccessPointMethodInitialize(&mi))

KUint32 LMI_RemoteServiceAccessPoint_RequestStateChange(
    const CMPIBroker* cb,
    CMPIMethodMI* mi,
    const CMPIContext* context,
    const LMI_RemoteServiceAccessPointRef* self,
    const KUint16* RequestedState,
    KRef* Job,
    const KDateTime* TimeoutPeriod,
    CMPIStatus* status)
{
    KUint32 result = KUINT32_INIT;

    KSetStatus(status, ERR_NOT_SUPPORTED);
    return result;
}

KONKRET_REGISTRATION(
    "root/cimv2",
    "LMI_RemoteServiceAccessPoint",
    "LMI_RemoteServiceAccessPoint",
    "instance method")
