/*
 * Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Radek Novacek <rnovacek@redhat.com>
 */

#include "globals.h"

#include <string.h>
#include <unistd.h>

#include <uuid/uuid.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/utsname.h>
#include <cmpimacs.h>

static char *_fqdn = NULL;
#ifdef DEBUG_ENABLED
static int _log_level = DEBUG;
#else
static int _log_level = WARN;
#endif

static const CMPIBroker *_cb = NULL;

#define LOG_ID "openlmi-networking"

char *getFQDN(void)
{
    struct utsname uts;
    if ((uname(&uts) > 0) && (uts.nodename != NULL)) {
        return strdup(uts.nodename);
    }
    char hostname[256];
    hostname[255] = '\0';
    if (gethostname(hostname, 255) == -1) {
        // FIXME: what to do, if we can't use gethostname?
        return NULL;
    }

    struct addrinfo hints;
    struct addrinfo *info = NULL, *p;
    memset(&hints, 0, sizeof hints);
    hints.ai_family = AF_UNSPEC; // either IPV4 or IPV6
    hints.ai_socktype = SOCK_STREAM;
    hints.ai_flags = AI_CANONNAME;

    if (getaddrinfo(hostname, "http", &hints, &info) == 0) {
        for (p = info; p != NULL; p = p->ai_next) {
            if (p->ai_canonname && strstr(p->ai_canonname, "localhost") == NULL) {
                char *dn = strdup(p->ai_canonname);
                freeaddrinfo(info);
                return dn;
            }
        }
    }
    if (info != NULL) {
        freeaddrinfo(info);
    }
    return strdup(hostname);
}

const char *get_system_name()
{
    if (_fqdn == NULL) {
        _fqdn = getFQDN();
    }
    return _fqdn;
}

const char *get_system_creationg_class_name()
{
    return "Linux_ComputerSystem";
}

GQuark cim_error_quark(void)
{
    return g_quark_from_static_string("cim-error-quark");
}

char *ip4ToString(uint32_t i)
{
    struct in_addr in = { i };
    char *ip = malloc(INET_ADDRSTRLEN);
    if (inet_ntop(AF_INET, &in, ip, INET_ADDRSTRLEN) == NULL) {
        free(ip);
        return NULL;
    }
    return ip;
}

uint32_t ip4FromString(const char *ip)
{
    if (ip == NULL) {
        error("Invalid argument (null) for ip4FromString");
        return 0;
    }
    struct in_addr inp;
    if (inet_pton(AF_INET, ip, &inp) <= 0) {
        warn("IPv4 address %s is not valid.", ip);
        return 0;
    }
    return inp.s_addr;
}

char *ip6ToString(const struct in6_addr *i)
{
    if (i == NULL) {
        error("Invalid argument (null) for ip6ToString");
        return NULL;
    }
    char *ip = malloc(INET6_ADDRSTRLEN);
    if (inet_ntop(AF_INET6, i, ip, INET6_ADDRSTRLEN) == NULL) {
        free(ip);
        return NULL;
    }
    return ip;

}

struct in6_addr *ip6FromString(const char *ip)
{
    if (ip == NULL) {
        error("Invalid argument (null) for ip6FromString");
        return NULL;
    }
    struct in6_addr *inp = malloc(sizeof(struct in6_addr));
    if (inet_pton(AF_INET6, ip, inp) <= 0) {
        warn("IPv6 address %s is not valid.", ip);
        return NULL;
    }
    return inp;
}

char *prefixToMask4(uint8_t prefix)
{
    return ip4ToString(((1 << prefix) - 1));
}

uint8_t netmaskToPrefix4(const char *netmask)
{
    if (netmask == NULL) {
        error("Invalid argument (null) for netmaskToPrefix4");
        return 0;
    }
    struct in_addr inp;
    if (inet_pton(AF_INET, netmask, &inp) <= 0) {
        warn("Invalid netmask: %s", netmask);
        return 0;
    }
    uint8_t ret = 0;
    while (inp.s_addr != 0) {
        ret += (inp.s_addr & 0x01);
        inp.s_addr >>= 1;
    }
    return ret;
}

char *macFromGByteArray(const GByteArray *mac)
{
    if (mac == NULL) {
        error("Invalid argument (null) for macFromGByteArray");
        return NULL;
    }
    char *m;
    asprintf(&m, "%.2X:%.2X:%.2X:%.2X:%.2X:%.2X",
             (int) mac->data[0], (int) mac->data[1],
             (int) mac->data[2], (int) mac->data[3],
             (int) mac->data[4], (int) mac->data[5]);
    return m;
}

GByteArray *macToGByteArray(const char *mac)
{
    if (mac == NULL) {
        error("Invalid argument (null) for macToGByteArray");
        return NULL;
    }
    unsigned int m[6];
    uint8_t mm[6];
    if (sscanf(mac, "%2X:%2X:%2X:%2X:%2X:%2X", &m[0], &m[1], &m[2], &m[3], &m[4], &m[5]) != 6) {
        error("MAC address is not valid: %s", mac);
        return NULL;
    }
    for (unsigned int i = 0; i < 6; ++i) {
        if (m[i] > 255) {
            error("MAC address is not valid: %s", mac);
            return NULL;
        }
        mm[i] = (uint8_t) m[i];
    }
    GByteArray *array = g_byte_array_sized_new(6);
    g_byte_array_append(array, mm, 6);
    return array;
}

char *uuid_gen(void)
{
    uuid_t uuid;
    char *buf = malloc(37 * sizeof(char));
    uuid_generate_random(uuid);
    uuid_unparse_lower(uuid, &buf[0]);
    return buf;
}

void init_logging(const CMPIBroker *cb)
{
    _cb = cb;
}

int log_level(void)
{
    return _log_level;
}

void set_log_level(int level)
{
    _log_level = level;
}

void _debug(int level, const char *file, int line, const char *format, ...)
{
    const char *lvl[] = { "NONE", "ERROR", "INFO", "WARNING", "DEBUG" };
    if (level > 4) {
        level = 4;
    }
    if (level < 1) {
        level = 1;
    }

    va_list args;
    va_start(args, format);

    CMPIStatus rc;

    if (_cb != NULL) {
        // try to use standard CMPI logging
        char *text1, *text2;
        vasprintf(&text1, format, args);
        asprintf(&text2, "[%s] %s:%d\t%s", lvl[level], file, line, text1);
        rc = _cb->eft->trace(_cb, CMPI_LEV_INFO, LOG_ID, text2, NULL);
        free(text1);
        free(text2);
    }

    if (_cb == NULL || rc.rc != CMPI_RC_OK) {
        if (level > _log_level) {
            va_end(args);
            return;
        }

        fprintf(stderr, "[%s] %s:%d\t", lvl[level], file, line);
        vfprintf(stderr, format, args);
        fprintf(stderr, "\n");
    }
    va_end(args);
}
