/*
 * Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Radek Novacek <rnovacek@redhat.com>
 */

#ifndef GLOBALS_H
#define GLOBALS_H

struct in6_addr;

// Unique identification of organization, should be trademarked, but meh.
#define ORGID "LMI"

#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>
#include <stdbool.h>
#include <glib.h>
#include <assert.h>
#include <cmpidt.h>

typedef struct Network Network;
typedef struct Port Port;
typedef struct Ports Ports;
typedef struct PortStat PortStat;
typedef struct PortStats PortStats;
typedef struct Endpoint Endpoint;
typedef struct Endpoints Endpoints;
typedef struct Connection Connection;
typedef struct Connections Connections;
typedef struct ActiveConnection ActiveConnection;
typedef struct ActiveConnections ActiveConnections;
typedef struct Setting Setting;
typedef struct Settings Settings;

typedef enum ProtocolType { Unknown=0, IPv4=1, IPv6=2, IPv4_IPv6=3 } ProtocolType;

typedef enum SettingType {
    SETTING_UNKNOWN=0,
    SETTING_STATIC,
    SETTING_LINK_LOCAL,
    SETTING_DHCP
} SettingType;

const char *get_system_name();

const char *get_system_creationg_class_name();

char *ip4ToString(uint32_t i);
uint32_t ip4FromString(const char *ip);

char *ip6ToString(const struct in6_addr *i);
struct in6_addr *ip6FromString(const char *);

char *prefixToMask4(uint8_t prefix);
uint8_t netmaskToPrefix4(const char *);

char *macFromGByteArray(const GByteArray *mac);
GByteArray *macToGByteArray(const char *mac);

/**
 * Generate random UUID
 *
 * \return random UUID (caller is responsibe to free it)
 */
char *uuid_gen(void);

enum { NONE=0, ERROR, INFO, WARN, DEBUG };

int log_level(void);
void set_log_level(int level);

/**
 * To use standard CIMOM logging facility, broker must be assigned.
 *
 * \p cb CMPIBroker
 */
void init_logging(const CMPIBroker *cb);

void _debug(int level, const char *file, int line, const char *format, ...);
#define debug(...) _debug(DEBUG, __FILE__, __LINE__, __VA_ARGS__)
#define warn(...)  _debug(WARN,  __FILE__, __LINE__, __VA_ARGS__)
#define info(...)  _debug(INFO,  __FILE__, __LINE__, __VA_ARGS__)
#define error(...) _debug(ERROR, __FILE__, __LINE__, __VA_ARGS__)

#define foreach(item, list, items)                                             \
    for (size_t i = 0, item = items##_index(list, 0);                          \
         i < items##_length(list);                                             \
         item = items##_index(list, i++))

#define LIST_DECL(Item, item)                                                  \
Item##s *item##s_new(size_t preallocated);                                     \
int item##s_add(Item##s *items, Item *item);                                   \
Item *item##s_index(const Item##s *items, size_t index);                       \
size_t item##s_length(const Item##s *items);                                   \
Item *item##s_pop(Item##s *items, size_t index);                               \
void item##s_free(Item##s *items, bool deep);

#define LIST_IMPL(Item, item)                                                  \
struct Item##s {                                                               \
    Item **items;                                                              \
    size_t length;                                                             \
    size_t allocated;                                                          \
};                                                                             \
                                                                               \
Item##s *item##s_new(size_t preallocated)                                      \
{                                                                              \
    Item##s *items = malloc(sizeof(Item##s));                                  \
    if (preallocated > SIZE_MAX / sizeof(Item *)) {                            \
        error("Malloc overflow detected");                                     \
        return NULL;                                                           \
    }                                                                          \
    items->items = malloc(preallocated * sizeof(Item *));                      \
    if (items->items == NULL) {                                                \
        error("Malloc failed");                                                \
        free(items);                                                           \
        return NULL;                                                           \
    }                                                                          \
    items->allocated = preallocated;                                           \
    items->length = 0;                                                         \
    return items;                                                              \
}                                                                              \
                                                                               \
int item##s_add(Item##s *items, Item *item)                                    \
{                                                                              \
    if (items->allocated <= items->length) {                                   \
        items->allocated += 5;                                                 \
        if (items->allocated > SIZE_MAX / sizeof(Item *)) {                    \
            error("Realloc overflow detected");                                \
            return 1;                                                          \
        }                                                                      \
        void *tmp = realloc(items->items, items->allocated * sizeof(Item *));  \
        if (tmp == NULL) {                                                     \
            error("Realloc failed");                                           \
            return 1;                                                          \
        }                                                                      \
        items->items = tmp;                                                    \
    }                                                                          \
    items->items[items->length++] = item;                                      \
    return 0;                                                                  \
}                                                                              \
                                                                               \
Item *item##s_index(const Item##s *items, size_t index)                        \
{                                                                              \
    if (index < items->length)                                                 \
        return items->items[index];                                            \
    else                                                                       \
        return NULL;                                                           \
}                                                                              \
                                                                               \
size_t item##s_length(const Item##s *items)                                    \
{                                                                              \
    if (items == NULL)                                                         \
        return 0;                                                              \
    return items->length;                                                      \
}                                                                              \
                                                                               \
Item *item##s_pop(Item##s *items, size_t index)                                \
{                                                                              \
    if (index >= items->length) {                                              \
        return NULL;                                                           \
    }                                                                          \
    Item *item = items->items[index];                                          \
    for (size_t i = index; i < items->length - 1; ++i) {                       \
        items->items[i] = items->items[i + 1];                                 \
    }                                                                          \
    items->length--;                                                           \
    return item;                                                               \
}                                                                              \
                                                                               \
void item##s_free(Item##s *items, bool deep)                                   \
{                                                                              \
    if (deep) {                                                                \
        for (size_t i = 0; i < items->length; ++i) {                           \
            item##_free(items->items[i]);                                      \
        }                                                                      \
    }                                                                          \
    free(items->items);                                                        \
    free(items);                                                               \
}


/**
 * Test if \p Variable is not null and exists.
 * Show \p Error and return if not.
 */
#define Require(Variable, Error)                            \
if (Variable->null || !Variable->exists) {                  \
    error(Error);                                           \
    KSetStatus2(_cb, status, ERR_INVALID_PARAMETER, Error); \
    return result;                                          \
}

#endif
