/*
 * Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Radek Novacek <rnovacek@redhat.com>
 */

#include "network_nm.h"

#include <string.h>
#include <unistd.h>

#include "activeconnection.h"
#include "dbus_wrapper.h"
#include "errors.h"

#include "activeconnection_nm.h"
#include "connection_nm.h"
#include "port_nm.h"

#include "network_private.h"
#include "port_private.h"
#include "nm_support.h"
#include "connection_private.h"

int DBUS_BUS = DBUS_BUS_SYSTEM;
const char *NM_SERVICE_DBUS = NM_DBUS_SERVICE;

typedef struct _NetworkPriv {
    DBusGConnection *connection;
    DBusGProxy *managerProxy;
    DBusGProxy *connectionProxy;
    GHashTable *properties;
} NetworkPriv;

void device_added_cb(void *proxy, const char *objectpath, Network *network);
void device_removed_cb(void *proxy, const char *objectpath, Network *network);
void manager_properties_changed_cb(void *proxy, GHashTable *properties, Network *network);
void manager_state_changed_cb(void *proxy, unsigned int state, Network *network);
void connection_added_cb(void *proxy, const char *objectpath, Network *network);
void connection_properties_changed_cb(void *proxy, GHashTable *properties, Network *network);
void network_priv_get_connections(Network *network);

void *network_priv_new(Network *network)
{
    // network is already locked in the caller
    g_type_init();
    GError *err = NULL;
    NetworkPriv *priv = malloc(sizeof(NetworkPriv));
    network->priv = priv;

    // Necessary for DBus to work in separate thread
    dbus_g_thread_init();

    // For testing purposes it's possible to use fake NM DBus API
    char *env = getenv("LMI_NETWORKING_FAKE_NM");
    if (env != NULL && strcmp(env, "1") == 0) {
        DBUS_BUS = DBUS_BUS_SESSION;
        NM_SERVICE_DBUS = "org.freedesktop.FakeNetworkManager";
    }

    // Connect to DBus
    priv->connection = dbus_g_bus_get(DBUS_BUS, &err);
    if (priv->connection == NULL) {
        error("Failed to open connection to bus: %s\n", err->message);
        return NULL;
    }
    priv->managerProxy = dbus_g_proxy_new_for_name(priv->connection, NM_SERVICE_DBUS, NM_DBUS_PATH, NM_DBUS_INTERFACE);
    priv->connectionProxy = dbus_g_proxy_new_for_name(priv->connection, NM_SERVICE_DBUS, NM_DBUS_PATH_SETTINGS, NM_DBUS_IFACE_SETTINGS);

    // Read manager properties
    priv->properties = dbus_get_properties(priv->managerProxy, NM_DBUS_PATH, NM_DBUS_INTERFACE);

    // Read ports, connections and active connection from NetworkManager
    network_priv_get_devices(network);
    network_priv_get_connections(network);
    network_priv_get_active_connections(network);

    dbus_g_proxy_add_signal(priv->managerProxy, "DeviceAdded", G_TYPE_STRING, G_TYPE_INVALID);
    dbus_g_proxy_connect_signal(priv->managerProxy, "DeviceAdded", G_CALLBACK(device_added_cb), network, NULL);

    dbus_g_proxy_add_signal(priv->managerProxy, "DeviceRemoved", DBUS_TYPE_G_OBJECT_PATH, G_TYPE_INVALID);
    dbus_g_proxy_connect_signal(priv->managerProxy, "DeviceRemoved", G_CALLBACK(device_removed_cb), network, NULL);

    dbus_g_proxy_add_signal(priv->managerProxy, "PropertiesChanged", DBUS_TYPE_G_MAP_OF_VARIANT, G_TYPE_INVALID);
    dbus_g_proxy_connect_signal(priv->managerProxy, "PropertiesChanged", G_CALLBACK(manager_properties_changed_cb), network, NULL);

    dbus_g_proxy_add_signal(priv->managerProxy, "StateChanged", G_TYPE_UINT, G_TYPE_INVALID);
    dbus_g_proxy_connect_signal(priv->managerProxy, "StateChanged", G_CALLBACK(manager_state_changed_cb), network, NULL);

    dbus_g_proxy_add_signal(priv->connectionProxy, "NewConnection", DBUS_TYPE_G_OBJECT_PATH, G_TYPE_INVALID);
    dbus_g_proxy_connect_signal(priv->connectionProxy, "NewConnection", G_CALLBACK(connection_added_cb), network, NULL);

    dbus_g_proxy_add_signal(priv->connectionProxy, "PropertiesChanged", DBUS_TYPE_G_MAP_OF_VARIANT, G_TYPE_INVALID);
    dbus_g_proxy_connect_signal(priv->connectionProxy, "PropertiesChanged", G_CALLBACK(connection_properties_changed_cb), network, NULL);

    return priv;
}

DBusGConnection *network_priv_get_dbus_connection(Network *network)
{
    NetworkPriv *priv = network->priv;
    return priv->connection;
}

void network_priv_free(void *data)
{
    NetworkPriv *priv = (NetworkPriv *) data;
    dbus_g_connection_unref(priv->connection);
    g_object_unref(priv->managerProxy);
    g_object_unref(priv->connectionProxy);
    g_hash_table_destroy(priv->properties);
    free(priv);
}

int network_priv_activate_connection(Network *network, const Port *port, const Connection *connection)
{
    NetworkPriv *priv = network->priv;
    GError *err = NULL;
    char *activeConnection;
    if (!dbus_g_proxy_call(priv->managerProxy, "ActivateConnection", &err,
            DBUS_TYPE_G_OBJECT_PATH, connection->uuid,
            DBUS_TYPE_G_OBJECT_PATH, port_get_uuid(port),
            DBUS_TYPE_G_OBJECT_PATH, "/", G_TYPE_INVALID,
            DBUS_TYPE_G_OBJECT_PATH, &activeConnection, G_TYPE_INVALID)) {

        error("Unable to activate connection %s on port %s: %s",
              connection ? connection_get_name(connection) : "NULL",
              port ? port_get_id(port) : "NULL",
              err->message);

        char *error_name = err->message + strlen(err->message) + 1;
        if (strcmp(error_name, "org.freedesktop.NetworkManager.Error.UnknownConnection") == 0) {
            return LMI_ERROR_CONNECTION_UNKNOWN;
        } else if (strcmp(error_name, "org.freedesktop.NetworkManager.Error.UnknownDevice") == 0) {
            return LMI_ERROR_PORT_UNKNOWN;
        } else if (strcmp(error_name, "org.freedesktop.NetworkManager.Error.ConnectionActivating") == 0) {
            return LMI_ERROR_CONNECTION_ACTIVATING;
        } else if (strcmp(error_name, "org.freedesktop.NetworkManager.Error.ConnectionInvalid") == 0) {
            return LMI_ERROR_CONNECTION_INVALID;
        } else {
            return LMI_ERROR_UNKNOWN;
        }
    }

    GValue *v;
    unsigned int state;
    for (unsigned int i = 0; i < 10; ++i) {
        debug("Waiting for connection %s to activate", activeConnection);
        v = dbus_get_property(priv->managerProxy, activeConnection, NM_DBUS_INTERFACE_ACTIVE_CONNECTION, "State");
        if (v) {
            state = g_value_get_uint(v);
            if (state == NM_ACTIVE_CONNECTION_STATE_ACTIVATED) {
                return LMI_SUCCESS;
            }
        }
        usleep(500000);
    }
    return LMI_ERROR_TIMEOUT;
}

Connection *network_priv_get_default_connection(Network *network, Port *port)
{
    Connection *connection;
    const Connections *connections = network->connections;
    for (size_t i = 0; i < connections_length(connections); ++i) {
        connection = connections_index(connections, i);
        if (!connection->autoconnect) {
            continue;
        }
        if (connection->port && port_compare(connection->port, port)) {
            return connection;
        }
    }
    error("Port %s has no default connection.", port_get_id(port));
    return NULL;
}

Endpoints *device_get_endpoints(Network *network, Port *port)
{
    return port->endpoints;
}

void device_added_cb(void *proxy, const char *objectpath, Network *network)
{
    debug("Device added: %s", objectpath);
    MUTEX_LOCK(network);
    Port *port = port_new_from_objectpath(network, objectpath);
    port->endpoints = device_get_endpoints(network, port);
    ports_add(network->ports, port);
    MUTEX_UNLOCK(network);
}

void device_removed_cb(void *proxy, const char *objectpath, Network *network)
{
    debug("Device removed: %s", objectpath);
    MUTEX_LOCK(network);
    size_t i, len = ports_length(network->ports);
    for (i = 0; i < len; i++) {
        if (strcmp(ports_index(network->ports, i)->uuid, objectpath) == 0) {
            break;
        }
    }
    if (i < len) {
        port_free(ports_pop(network->ports, i));
    }
    MUTEX_UNLOCK(network);
}

void connection_added_cb(void *proxy, const char *objectpath, Network *network)
{
    debug("Connection added: %s", objectpath);
    MUTEX_LOCK(network);
    Connection *connection = connection_new_from_objectpath(network, objectpath);
    connections_add(network->connections, connection);
    MUTEX_UNLOCK(network);
}

void connection_properties_changed_cb(void *proxy, GHashTable *properties, Network *network)
{
    warn("Connection properties changed - not implemented");
    GHashTableIter iter;
    gpointer key, value;
    g_hash_table_iter_init(&iter, properties);
    while (g_hash_table_iter_next(&iter, &key, &value)) {
        debug("\tProperty changed: %s", (char *) key);
    }
    debug("---");
}

void manager_properties_changed_cb(void *proxy, GHashTable *properties, Network *network)
{
    NetworkPriv *priv = network->priv;
    GHashTableIter iter;
    g_hash_table_iter_init(&iter, properties);
    gpointer key, value;
    MUTEX_LOCK(network);
    while (g_hash_table_iter_next(&iter, &key, &value)) {
        if (strcmp((char *) key, "ActiveConnections") == 0) {
            debug("Active Connections changed");
            // Reload active connections
            priv->properties = dbus_get_properties(priv->managerProxy, NM_DBUS_PATH, NM_DBUS_INTERFACE);
            network_priv_get_active_connections(network);
        } else {
            debug("Manager - unhandled property changed: %s", (char *) key);
        }
    }
    MUTEX_UNLOCK(network);
}

void manager_state_changed_cb(void *proxy, unsigned int state, Network *network)
{
    debug("Manager state changed: %d - not implemented", state);
}

void network_priv_get_devices(Network *network)
{
    // network is already locked in the caller
    NetworkPriv *priv = network->priv;

    char *device;
    GPtrArray *devices;
    GError *err = NULL;
    if (!dbus_g_proxy_call(priv->managerProxy, "GetDevices", &err, G_TYPE_INVALID,
        DBUS_TYPE_G_ARRAY_OF_OBJECT_PATH, &devices, G_TYPE_INVALID)) {

        error("Calling method GetDevices failed: %s", err->message);
        return;
    }

    network->ports = ports_new(devices->len);

    Port *port;
    for (guint i = 0; i < devices->len; ++i) {
        device = devices->pdata[i];
        debug("Device: %s", device);
        port = port_new_from_objectpath(network, device);
        port->endpoints = device_get_endpoints(network, port);
        ports_add(network->ports, port);

        free(device);
    }
    g_ptr_array_free(devices, TRUE);
}

void network_priv_get_connections(Network *network)
{
    // network is already locked in the caller
    NetworkPriv *priv = network->priv;
    GPtrArray *connections;
    GError *err = NULL;

    if (!dbus_g_proxy_call(priv->connectionProxy, "ListConnections", &err, G_TYPE_INVALID,
        dbus_g_type_get_collection("GPtrArray", DBUS_TYPE_G_OBJECT_PATH), &connections, G_TYPE_INVALID)) {

        error("Calling method ListConnections failed: %s", err->message);
        return;
    }

    network->connections = connections_new(connections->len);

    Connection *connection;
    char *path;
    for (guint i = 0; i < connections->len; ++i) {
        path = connections->pdata[i];
        debug("Connection: %s", path);
        connection = connection_new_from_objectpath(network, path);
        connections_add(network->connections, connection);
        free(path);
    }
    g_ptr_array_free(connections, TRUE);
}

void network_priv_get_active_connections(Network *network)
{
    NetworkPriv *priv = network->priv;
    GPtrArray *array = dbus_property_array(priv->properties, "ActiveConnections");

    if (array != NULL) {
        network->activeConnections = active_connections_new(array->len);

        const char *objectpath;
        ActiveConnection *activeConnection;
        for (guint i = 0; i < array->len; ++i) {
            objectpath = (char *) g_ptr_array_index(array, i);
            activeConnection = active_connection_from_objectpath(network, objectpath);
            if (activeConnection == NULL) {
                continue;
            }
            active_connections_add(network->activeConnections, activeConnection);
        }
    } else {
        network->activeConnections = active_connections_new(0);
    }
}

int network_priv_create_connection(Network *network, Connection *connection)
{
    debug("network_priv_create_connection");
    NetworkPriv *priv = network->priv;
    char *objectpath;
    GError *err = NULL;

    GHashTable *hash = connection_to_hash(connection);
    check_connection_hash(hash);

    int res = 0;
    if (!dbus_g_proxy_call(priv->connectionProxy, "AddConnection", &err,
            DBUS_TYPE_G_MAP_OF_MAP_OF_VARIANT, hash, G_TYPE_INVALID,
            DBUS_TYPE_G_OBJECT_PATH, &objectpath, G_TYPE_INVALID)) {

        error("Creating of connection failed: %d %s", err->code, err->message);
        res = 1;
    }
    connection->uuid = objectpath;
    g_hash_table_destroy(hash);
    return res;
}

int network_priv_delete_connection(Network *network, Connection *connection)
{
    return connection_delete(connection);
}
