/*
 * Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Radek Novacek <rnovacek@redhat.com>
 */

#include "nm_support.h"

#include "globals.h"
#include "dbus_wrapper.h"

#include <stdbool.h>
#include <dbus/dbus-glib.h>
#include <string.h>


gboolean ipv4_address_from_array(GArray *array, guint32 *address, guint32 *prefix, guint32 *gateway)
{
    if (array->len < 2 || array->len > 3) {
        return false;
    }

    *address = g_array_index(array, guint32, 0);
    *prefix =  g_array_index(array, guint32, 1);
    if (array->len > 2) {
        *gateway = g_array_index(array, guint32, 2);
    } else {
        *gateway = 0;
    }
    return true;
}

GArray *ipv4_address_to_array(guint32 address, guint32 prefix, guint32 gateway)
{
    GArray *tmp = g_array_new(FALSE, FALSE, sizeof(guint32));
    g_array_append_val(tmp, address);
    g_array_append_val(tmp, prefix);
    g_array_append_val(tmp, gateway);
    return tmp;
}

gboolean ipv6_address_from_array(GValueArray *array, GByteArray **address, guint32 *prefix, GByteArray **gateway)
{
    GValue *tmp = NULL;

    if (array->n_values < 2 || array->n_values > 3) {
        return false;
    }

    // address
    tmp = g_value_array_get_nth(array, 0);
    if (G_VALUE_TYPE(tmp) != DBUS_TYPE_G_UCHAR_ARRAY) {
        return false;
    }
    *address = g_value_get_boxed(tmp);
    if ((*address)->len != 16) {
        return false;
    }

    tmp = g_value_array_get_nth(array, 1);
    if (G_VALUE_TYPE(tmp) != G_TYPE_UINT) {
        return false;
    }
    *prefix = g_value_get_uint(tmp);
    if (*prefix > 128) {
        return false;
    }

    /* Check optional 3rd element (gateway) */
    if (array->n_values > 2) {
        tmp = g_value_array_get_nth(array, 2);
        if (G_VALUE_TYPE(tmp) != DBUS_TYPE_G_UCHAR_ARRAY) {
            return false;
        }
        *gateway = g_value_get_boxed(tmp);
        if ((*gateway)->len != 16) {
            return false;
        }
    }
    return true;
}

GValueArray *ipv6_address_to_array(GByteArray *address, guint32 prefix, GByteArray *gateway)
{
    GValue element = G_VALUE_INIT;
    GValueArray *tmp = g_value_array_new(2);

    g_value_init(&element, DBUS_TYPE_G_UCHAR_ARRAY);
    g_value_take_boxed(&element, address);
    g_value_array_append(tmp, &element);
    g_value_unset(&element);

    g_value_init(&element, G_TYPE_UINT);
    g_value_set_uint(&element, prefix);
    g_value_array_append(tmp, &element);
    g_value_unset(&element);

    if (gateway != NULL) {
        g_value_init(&element, DBUS_TYPE_G_UCHAR_ARRAY);
        g_value_take_boxed(&element, gateway);
        g_value_array_append(tmp, &element);
        g_value_unset(&element);
    }
    return tmp;
}

void g_hash_table_print(GHashTable *hash, gboolean first)
{
    GHashTableIter iter;
    g_hash_table_iter_init(&iter, hash);
    gpointer key, value;
    GValue *v;
    fprintf(stderr, "[\n");
    while (g_hash_table_iter_next(&iter, &key, &value)) {
        fprintf(stderr, "\t%s: ", (char *) key);
        if (!first) {
            v = value;
            if (v) {
                if (G_VALUE_HOLDS_STRING(v)) {
                    fprintf(stderr, "\"%s\"\n", g_value_get_string(v));
                } else if (G_VALUE_HOLDS(v, DBUS_TYPE_G_UCHAR_ARRAY)) {
                    GByteArray *a = g_value_get_boxed(v);
                    fprintf(stderr, "\"");
                    for (guint i = 0; i < a->len; ++i) {
                        fprintf(stderr, "%.2X", a->data[i]);
                        if (i < a->len - 1) {
                            fprintf(stderr, ":");
                        }
                    }
                    fprintf(stderr, "\"\n");
                } else {
                    fprintf(stderr, "<unknown type: %s>\n", G_VALUE_TYPE_NAME(v));
                }
            } else {
                fprintf(stderr, "Value is not GValue\n");
            }
        } else {
            g_hash_table_print(value, false);
        }
    }
    fprintf(stderr, "]\n");
}

void assert_gvalue_key(const char *key, const char *subkey, GValue *v, GType t)
{
    if (!G_VALUE_HOLDS(v, t)) {
        error("Wrong type for key \"%s.%s\": %s", key, subkey, G_VALUE_TYPE_NAME(v));
    }
}

void check_connection_hash(GHashTable* hash)
{
    debug("check_connection_hash");
    GHashTable *subhash;
    GHashTableIter iter, subiter;
    g_hash_table_iter_init(&iter, hash);
    gpointer key, subkey, value;
    while (g_hash_table_iter_next(&iter, &key, &value)) {
        subhash = value;
        g_hash_table_iter_init(&subiter, subhash);
        while (g_hash_table_iter_next(&subiter, &subkey, &value)) {
            if (strcmp((char *) key, "connection") == 0) {
                if (strcmp((char *) subkey, "autoconnect") == 0) {
                    assert_gvalue_key(key, subkey, value, G_TYPE_BOOLEAN);
                } else if (strcmp((char *) subkey, "id") == 0) {
                    assert_gvalue_key(key, subkey, value, G_TYPE_STRING);
                } else if (strcmp((char *) subkey, "type") == 0) {
                    assert_gvalue_key(key, subkey, value, G_TYPE_STRING);
                } else if (strcmp((char *) subkey, "uuid") == 0) {
                    assert_gvalue_key(key, subkey, value, G_TYPE_STRING);
                }
            } else if (strcmp((char *) key, "ipv4") == 0) {
                if (strcmp((char *) subkey, "method") == 0) {
                    assert_gvalue_key(key, subkey, value, G_TYPE_STRING);
                } else if (strcmp((char *) subkey, "addresses") == 0) {
                    assert_gvalue_key(key, subkey, value, DBUS_TYPE_G_ARRAY_OF_ARRAY_OF_UINT);
                } else if (strcmp((char *) subkey, "dns") == 0) {
                    assert_gvalue_key(key, subkey, value, DBUS_TYPE_G_ARRAY_OF_UINT);
                } else if (strcmp((char *) subkey, "routes") == 0) {
                    assert_gvalue_key(key, subkey, value, DBUS_TYPE_G_ARRAY_OF_ARRAY_OF_UINT);
                }
            } else if (strcmp((char *) key, "ipv6") == 0) {
                if (strcmp((char *) subkey, "method") == 0) {
                    assert_gvalue_key(key, subkey, value, G_TYPE_STRING);
                } else if (strcmp((char *) subkey, "addresses") == 0) {
                    assert_gvalue_key(key, subkey, value, DBUS_TYPE_G_ARRAY_OF_IP6_ADDRESS);
                } else if (strcmp((char *) subkey, "dns") == 0) {
                    assert_gvalue_key(key, subkey, value, DBUS_TYPE_G_ARRAY_OF_ARRAY_OF_UCHAR);
                } else if (strcmp((char *) subkey, "routes") == 0) {
                    assert_gvalue_key(key, subkey, value, DBUS_TYPE_G_ARRAY_OF_IP6_ROUTE);
                }
            } else if (strcmp((char *) key, "802-3-ethernet") == 0) {
                if (strcmp((char *) subkey, "mac-address") == 0) {
                    assert_gvalue_key(key, subkey, value, DBUS_TYPE_G_UCHAR_ARRAY);
                }
            } else {
                warn("Unknown connection key: %s", (char *) key);
            }
        }
    }
}
