/*
 * Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * Authors: Radek Novacek <rnovacek@redhat.com>
 */

#include <CIM_LANEndpoint.h>
#include <CIM_ServiceAccessPoint.h>
#include <CIM_NetworkPort.h>
#include <CIM_IPAssignmentSettingData.h>
#include <LMI_IPProtocolEndpoint.h>
#include <CIM_ComputerSystem.h>
#include <LMI_IPConfigurationService.h>
#include <LMI_IPAssignmentSettingData.h>
#include <LMI_RemoteServiceAccessPoint.h>
#include <CIM_LogicalDevice.h>
#include <LMI_DNSProtocolEndpoint.h>
#include "globals.h"
#include "port.h"
#include "endpoint.h"
#include "connection.h"
#include "setting.h"

CMPIObjectPath *getComputerSystemOP(const CMPIBroker *cb, const char *ns)
{
    CIM_ComputerSystemRef computerSystemRef;
    CIM_ComputerSystemRef_Init(&computerSystemRef, cb, ns);
    CIM_ComputerSystemRef_Set_Name(&computerSystemRef, get_system_name());
    CIM_ComputerSystemRef_Set_CreationClassName(&computerSystemRef, get_system_creationg_class_name());

    CMPIStatus rc;
    // Small hack - we need to adjust class name, because it won't get found without it
    CMPIObjectPath *computerSystemOP = CIM_ComputerSystemRef_ToObjectPath(&computerSystemRef, &rc);
    computerSystemOP->ft->setClassName(computerSystemOP, get_system_creationg_class_name());
    return computerSystemOP;
}

CMPIObjectPath *getIPConfigurationSystemOP(const CMPIBroker *cb, const char *ns)
{
    LMI_IPConfigurationServiceRef ipConfigurationServiceRef;
    LMI_IPConfigurationServiceRef_Init(&ipConfigurationServiceRef, cb, ns);
    LMI_IPConfigurationServiceRef_Set_CreationClassName(&ipConfigurationServiceRef, "LMI_IPConfigurationService");
    LMI_IPConfigurationServiceRef_Set_Name(&ipConfigurationServiceRef, "IPConfigurationService");
    LMI_IPConfigurationServiceRef_Set_SystemCreationClassName(&ipConfigurationServiceRef, get_system_creationg_class_name());
    LMI_IPConfigurationServiceRef_Set_SystemName(&ipConfigurationServiceRef, get_system_name());

    CMPIStatus rc;
    return LMI_IPConfigurationServiceRef_ToObjectPath(&ipConfigurationServiceRef, &rc);
}

CMPIObjectPath *portToCIM_LANEndpointRefOP(const Port *port, const CMPIBroker *cb, const char *ns)
{
    CIM_LANEndpointRef lanEndpointRef;
    CIM_LANEndpointRef_Init(&lanEndpointRef, cb, ns);
    CIM_LANEndpointRef_Set_CreationClassName(&lanEndpointRef, "LMI_LANEndpoint");
    CIM_LANEndpointRef_Set_Name(&lanEndpointRef, port_get_id(port));
    CIM_LANEndpointRef_Set_SystemCreationClassName(&lanEndpointRef, get_system_creationg_class_name());
    CIM_LANEndpointRef_Set_SystemName(&lanEndpointRef, get_system_name());

    CMPIStatus rc;
    CMPIObjectPath *lanEndpointOP = CIM_LANEndpointRef_ToObjectPath(&lanEndpointRef, &rc);
    lanEndpointOP->ft->setClassName(lanEndpointOP, "LMI_LANEndpoint");
    return lanEndpointOP;
}

CMPIObjectPath *portToCIM_NetworkPortRefOP(const Port *port, const CMPIBroker *cb, const char *ns)
{
    CIM_NetworkPortRef networkPortRef;
    CIM_NetworkPortRef_Init(&networkPortRef, cb, ns);
    CIM_NetworkPortRef_Set_CreationClassName(&networkPortRef, "LMI_EthernetPort");
    CIM_NetworkPortRef_Set_DeviceID(&networkPortRef, port_get_id(port));
    CIM_NetworkPortRef_Set_SystemCreationClassName(&networkPortRef, get_system_creationg_class_name());
    CIM_NetworkPortRef_Set_SystemName(&networkPortRef, get_system_name());

    CMPIStatus rc;
    CMPIObjectPath *networkPortOP = CIM_NetworkPortRef_ToObjectPath(&networkPortRef, &rc);
    networkPortOP->ft->setClassName(networkPortOP, "LMI_EthernetPort");
    return networkPortOP;
}

CMPIObjectPath *portToCIM_ServiceAccessPointRefOP(const Port *port, const CMPIBroker *cb, const char *ns)
{
    CIM_ServiceAccessPointRef serviceAccessPointRef;
    CIM_ServiceAccessPointRef_Init(&serviceAccessPointRef, cb, ns);
    CIM_ServiceAccessPointRef_Set_CreationClassName(&serviceAccessPointRef, "LMI_LANEndpoint");
    CIM_ServiceAccessPointRef_Set_Name(&serviceAccessPointRef, port_get_id(port));
    CIM_ServiceAccessPointRef_Set_SystemCreationClassName(&serviceAccessPointRef, get_system_creationg_class_name());
    CIM_ServiceAccessPointRef_Set_SystemName(&serviceAccessPointRef, get_system_name());

    CMPIStatus rc;
    CMPIObjectPath *serviceAccessPointOP = CIM_ServiceAccessPointRef_ToObjectPath(&serviceAccessPointRef, &rc);
    serviceAccessPointOP->ft->setClassName(serviceAccessPointOP, "LMI_LANEndpoint");
    return serviceAccessPointOP;
}

CMPIObjectPath *portToCIM_LogicalDeviceOP(const Port *port, const CMPIBroker *cb, const char *ns)
{
    CIM_LogicalDeviceRef logicalDeviceRef;
    CIM_LogicalDeviceRef_Init(&logicalDeviceRef, cb, ns);
    CIM_LogicalDeviceRef_Set_CreationClassName(&logicalDeviceRef, "LMI_EthernetPort");
    CIM_LogicalDeviceRef_Set_DeviceID(&logicalDeviceRef, port_get_id(port));
    CIM_LogicalDeviceRef_Set_SystemCreationClassName(&logicalDeviceRef, get_system_creationg_class_name());
    CIM_LogicalDeviceRef_Set_SystemName(&logicalDeviceRef, get_system_name());

    CMPIStatus rc;
    CMPIObjectPath *logicalDeviceOP = CIM_LogicalDeviceRef_ToObjectPath(&logicalDeviceRef, &rc);
    logicalDeviceOP->ft->setClassName(logicalDeviceOP, "LMI_EthernetPort");
    return logicalDeviceOP;
}

CMPIObjectPath *endpointToLMI_RemoteServiceAccessPointOP(const Endpoint *endpoint, const CMPIBroker *cb, const char *ns)
{
    LMI_RemoteServiceAccessPointRef remoteServiceAccessPointRef;
    LMI_RemoteServiceAccessPointRef_Init(&remoteServiceAccessPointRef, cb, ns);
    LMI_RemoteServiceAccessPointRef_Set_CreationClassName(&remoteServiceAccessPointRef, "LMI_RemoteServiceAccessPoint");
    LMI_RemoteServiceAccessPointRef_Set_Name(&remoteServiceAccessPointRef, endpoint_get_name(endpoint));
    LMI_RemoteServiceAccessPointRef_Set_SystemCreationClassName(&remoteServiceAccessPointRef, get_system_creationg_class_name());
    LMI_RemoteServiceAccessPointRef_Set_SystemName(&remoteServiceAccessPointRef, get_system_name());

    CMPIStatus rc;
    return LMI_RemoteServiceAccessPointRef_ToObjectPath(&remoteServiceAccessPointRef, &rc);
}

CMPIObjectPath *endpointToCIM_ServiceAccessPointRefOP(const Endpoint *endpoint, const CMPIBroker *cb, const char *ns)
{
    CIM_ServiceAccessPointRef serviceAccessPointRef;
    CIM_ServiceAccessPointRef_Init(&serviceAccessPointRef, cb, ns);
    CIM_ServiceAccessPointRef_Set_CreationClassName(&serviceAccessPointRef, "LMI_IPProtocolEndpoint");
    CIM_ServiceAccessPointRef_Set_Name(&serviceAccessPointRef, endpoint_get_name(endpoint));
    CIM_ServiceAccessPointRef_Set_SystemCreationClassName(&serviceAccessPointRef, get_system_creationg_class_name());
    CIM_ServiceAccessPointRef_Set_SystemName(&serviceAccessPointRef, get_system_name());

    CMPIStatus rc;
    CMPIObjectPath *serviceAccessPointOP = CIM_ServiceAccessPointRef_ToObjectPath(&serviceAccessPointRef, &rc);
    serviceAccessPointOP->ft->setClassName(serviceAccessPointOP, "LMI_IPProtocolEndpoint");
    return serviceAccessPointOP;
}

CMPIObjectPath *endpointToLMI_IPProtocolEndpointRefOP(const Endpoint *endpoint, const CMPIBroker *cb, const char *ns)
{
    LMI_IPProtocolEndpointRef ipProtocolEndpointRef;
    LMI_IPProtocolEndpointRef_Init(&ipProtocolEndpointRef, cb, ns);
    LMI_IPProtocolEndpointRef_Set_CreationClassName(&ipProtocolEndpointRef, "LMI_IPProtocolEndpoint");
    LMI_IPProtocolEndpointRef_Set_Name(&ipProtocolEndpointRef, endpoint_get_name(endpoint));
    LMI_IPProtocolEndpointRef_Set_SystemCreationClassName(&ipProtocolEndpointRef, get_system_creationg_class_name());
    LMI_IPProtocolEndpointRef_Set_SystemName(&ipProtocolEndpointRef, get_system_name());

    CMPIStatus rc;
    return LMI_IPProtocolEndpointRef_ToObjectPath(&ipProtocolEndpointRef, &rc);
}

CMPIObjectPath *endpointToLMI_DNSProtocolEndpointRefOP(const Endpoint *endpoint, const CMPIBroker *cb, const char *ns)
{
    LMI_DNSProtocolEndpointRef dnsProtocolEndpointRef;
    LMI_DNSProtocolEndpointRef_Init(&dnsProtocolEndpointRef, cb, ns);
    LMI_DNSProtocolEndpointRef_Set_CreationClassName(&dnsProtocolEndpointRef, "LMI_DNSProtocolEndpoint");
    LMI_DNSProtocolEndpointRef_Set_Name(&dnsProtocolEndpointRef, endpoint_get_name(endpoint));
    LMI_DNSProtocolEndpointRef_Set_SystemCreationClassName(&dnsProtocolEndpointRef, get_system_creationg_class_name());
    LMI_DNSProtocolEndpointRef_Set_SystemName(&dnsProtocolEndpointRef, get_system_name());

    CMPIStatus rc;
    return LMI_DNSProtocolEndpointRef_ToObjectPath(&dnsProtocolEndpointRef, &rc);
}

CMPIObjectPath *connectionToCIM_IPAssignmentSettingDataRefOP(const Connection *connection, const CMPIBroker *cb, const char *ns)
{
    CIM_IPAssignmentSettingDataRef settingDataRef_generic;
    CIM_IPAssignmentSettingDataRef_Init(&settingDataRef_generic, cb, ns);
    CIM_IPAssignmentSettingDataRef_Set_InstanceID(&settingDataRef_generic, connection_get_id(connection));

    CMPIStatus rc;
    CMPIObjectPath *settingDataOP = CIM_IPAssignmentSettingDataRef_ToObjectPath(&settingDataRef_generic, &rc);
    settingDataOP->ft->setClassName(settingDataOP, "LMI_IPAssignmentSettingData");
    return settingDataOP;

}

CMPIObjectPath *settingToLMI_IPAssignmentSettingDataSubclassOP(const Setting *setting, const CMPIBroker *cb, const char *ns)
{
    LMI_IPAssignmentSettingDataRef ipAssignmentSettingDataRef;
    LMI_IPAssignmentSettingDataRef_Init(&ipAssignmentSettingDataRef, cb, ns);
    const char *className;
    switch (setting_get_type(setting)) {
        case SETTING_STATIC:
            className = "LMI_StaticIPAssignmentSettingData";
            break;
        case SETTING_LINK_LOCAL:
            className = "LMI_StaticIPAssignmentSettingData";
            break;
        case SETTING_DHCP:
            className = "LMI_DHCPSettingData";
            break;
        default:
            warn("Unknown setting type: %d", setting_get_type(setting));
            break;
    }

    CMPIStatus rc;
    LMI_IPAssignmentSettingDataRef_Set_InstanceID(&ipAssignmentSettingDataRef, setting_get_id(setting));
    CMPIObjectPath *ipAssignmentSettingDataOP = LMI_IPAssignmentSettingDataRef_ToObjectPath(&ipAssignmentSettingDataRef, &rc);
    ipAssignmentSettingDataOP->ft->setClassName(ipAssignmentSettingDataOP, className);
    return ipAssignmentSettingDataOP;
}
